﻿using System;
using System.Collections.Generic;

namespace AdrianKousz.Util
{
	public class CmdApp
	{
		private string header;
		private IDictionary<string, Command> cmds;
		private Command noArgCommand;
		private Command defaultCommand;

		private CmdApp() { }

		public void Run(string[] args)
		{
			Console.WriteLine(header);
			Console.WriteLine();

			if (args.Length == 0) {
				noArgCommand.Run(new string[0]);
				return;
			}

			var cmdname = args[0];
			var cmd = GetCommand(cmdname);

			if (cmd == null) {
				defaultCommand.Run(new string[] { cmdname });
				return;
			}

			var requireargcount = cmd.MinArgumentCount;

			if (args.Length - 1 < requireargcount) {
				var hint = string.Format(SR.Error_ArgCount, cmdname, requireargcount);
				defaultCommand.Run(new string[] { cmdname, hint });
				return;
			}

			var cmdargs = new string[args.Length - 1];
			Array.Copy(args, 1, cmdargs, 0, cmdargs.Length);
			cmd.Run(cmdargs);
		}

		private Command GetCommand(string k)
		{
			k = k.ToLower();
			if (cmds.ContainsKey(k)) return cmds[k];
			return null;
		}

		public abstract class Command
		{
			public int MinArgumentCount { get; protected set; }
			public string Description { get; protected set; } = SR.NoDesc;
			public string Help { get; protected set; } = SR.NoHelp;
			public abstract void Run(string[] args);
		}

		public sealed class HelpCommand : Command
		{
			private CmdApp app;

			public HelpCommand(CmdApp app)
			{
				Description = SR.HelpCmdDesc;
				Help = SR.HelpCmdHelp;
				this.app = app;
			}

			public override void Run(string[] args)
			{
				var sb = new System.Text.StringBuilder();
				var showlist = true;
				var cmdname = args.Length > 0 ? args[0].ToLower() : "";

				if (args.Length > 1)
					Console.WriteLine(args[1]);

				if (!cmdname.Equals("")) {
					var cmd = app.GetCommand(cmdname);
					if (cmd == null) {
						sb.AppendFormat(SR.Error_NoCmd, cmdname);
						sb.AppendLine();
					} else {
						sb.AppendFormat(SR.HelpFor, cmdname);
						sb.AppendLine();
						sb.AppendLine();
						sb.AppendLine(cmd.Help);
						showlist = false;
					}
				}

				if (showlist) {
					sb.AppendLine(SR.AvailCmds);
					foreach (var cmd in app.cmds) {
						sb.AppendFormat("  {0,-12} {1}", cmd.Key, cmd.Value.Description);
						sb.AppendLine();
					}
				}

				Console.WriteLine(sb);

				if (args.Length == 0) {
					Console.WriteLine(SR.PressEnter);
					Console.ReadLine();
				}
			}
		}

		public sealed class Builder
		{
			private CmdApp result = new CmdApp();
			private Command helpCommand;

			public Builder()
			{
				result.cmds = new Dictionary<string, Command>();
				result.header = SR.DefaultAppHeader;
				helpCommand = new HelpCommand(result);
				result.noArgCommand = helpCommand;
				result.defaultCommand = helpCommand;
			}

			public Builder NoArg(Command value) { result.noArgCommand = value; return this; }
			public Builder Default(Command value) { result.defaultCommand = value; return this; }
			public Builder Header(string value) { result.header = value; return this; }

			public Builder Add(string name, Command command)
			{
				name = name.ToLower();
				if (name == null || name.Equals("") || name.Equals(SR.HelpCmdName))
					throw new ArgumentException(SR.Error_CmdNameForbidden);
				result.cmds.Add(name, command);
				return this;
			}

			public CmdApp Build()
			{
				result.cmds.Add(SR.HelpCmdName, helpCommand);
				var r = result;
				result = null;
				return r;
			}
		}

		private static class SR
		{
			public const string DefaultAppHeader = "Unnamed console application";
			public const string HelpCmdName = "help";
			public const string HelpCmdDesc = "Get help";
			public const string HelpCmdHelp = "List available commands or display help of a command\n\nParameters:\n[<command>]";
			public const string NoDesc = "(No short description available)";
			public const string NoHelp = "(No help available)";

			public const string PressEnter = "Press enter to continue ...";
			public const string HelpFor = "Help for {0}:";
			public const string AvailCmds = "Available commands are:";

			public const string Error_CmdNameForbidden = "Forbidden name";
			public const string Error_NoCmd = "\x1b[31mCommand \"{0}\" does not exist!\x1b[0m";
			public const string Error_ArgCount = "\x1b[31mCommand \"{0}\" requires at least {1} arguments!\x1b[0m";
		}
	}
}
