﻿using System;
using System.IO;

/**
 * About closing and disposing, also see:
 * http://joeduffyblog.com/2004/12/12/follow-up-should-you-invoke-close-andor-dispose-on-a-stream/
 * https://github.com/dotnet/corefx/blob/master/src/System.IO.Compression/src/System/IO/Compression/DeflateStream.cs
 */

namespace AdrianKousz.Util
{
	public sealed class NonDisposingStreamWrapper : Stream
	{
		public Stream BaseStream { get; private set; }

		public NonDisposingStreamWrapper(Stream stream)
		{
			BaseStream = stream;
		}

		protected override void Dispose(bool disposing)
		{
			BaseStream = null;
		}

		private void EnsureNotDisposed()
		{
			if (BaseStream == null) {
				throw new ObjectDisposedException(null);
			}
		}

		public override bool CanRead
		{
			get { return BaseStream != null && BaseStream.CanRead; }
		}

		public override bool CanSeek
		{
			get { return BaseStream != null && BaseStream.CanSeek; }
		}

		public override bool CanWrite
		{
			get { return BaseStream != null && BaseStream.CanWrite; }
		}

		public override long Length
		{
			get
			{
				EnsureNotDisposed();
				return BaseStream.Length;
			}
		}

		public override long Position
		{
			get
			{
				EnsureNotDisposed();
				return BaseStream.Position;
			}
			set
			{
				EnsureNotDisposed();
				BaseStream.Position = value;
			}
		}

		public override IAsyncResult BeginRead(byte[] buffer, int offset, int count,
			AsyncCallback callback, object state)
		{
			EnsureNotDisposed();
			return BaseStream.BeginRead(buffer, offset, count, callback, state);
		}

		public override IAsyncResult BeginWrite(byte[] buffer, int offset, int count,
			AsyncCallback callback, object state)
		{
			EnsureNotDisposed();
			return BaseStream.BeginWrite(buffer, offset, count, callback, state);
		}

		public override int EndRead(IAsyncResult asyncResult)
		{
			EnsureNotDisposed();
			return BaseStream.EndRead(asyncResult);
		}

		public override void EndWrite(IAsyncResult asyncResult)
		{
			EnsureNotDisposed();
			BaseStream.EndWrite(asyncResult);
		}

		public override void Flush()
		{
			EnsureNotDisposed();
			BaseStream.Flush();
		}

		public override int Read(byte[] buffer, int offset, int count)
		{
			EnsureNotDisposed();
			return BaseStream.Read(buffer, offset, count);
		}

		public override int ReadByte()
		{
			EnsureNotDisposed();
			return BaseStream.ReadByte();
		}

		public override long Seek(long offset, SeekOrigin origin)
		{
			EnsureNotDisposed();
			return BaseStream.Seek(offset, origin);
		}

		public override void SetLength(long value)
		{
			EnsureNotDisposed();
			BaseStream.SetLength(value);
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			EnsureNotDisposed();
			BaseStream.Write(buffer, offset, count);
		}

		public override void WriteByte(byte value)
		{
			EnsureNotDisposed();
			BaseStream.WriteByte(value);
		}
	}
}

