﻿using System;
using System.IO;

namespace AdrianKousz.Util
{
	public class UncompressedDeflateStream : Stream
	{
		private static readonly byte[] EOS = new byte[] {0x01, 0x00, 0x00, 0xff, 0xff};

		public Stream BaseStream { get; private set; }

		public UncompressedDeflateStream (Stream stream) : base()
		{
			BaseStream = stream;
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			EnsureNotDisposed();
			if (count > 65535) {
				throw new ArgumentOutOfRangeException("offset", "Larger than 65535");
			}

			var countHi = (count >> 8) & 0xff;
			var countLo = count & 0xff;

			var bytes = new byte[] { 0, (byte)countLo, (byte)countHi, (byte)~countLo, (byte)~countHi };

			BaseStream.Write(bytes, 0, bytes.Length);
			BaseStream.Write(buffer, offset, count);
		}

		protected override void Dispose(bool disposing)
		{
			try {
				if (disposing && BaseStream != null) {
					BaseStream.Write(EOS, 0, EOS.Length);
				}
			} finally {
				try {
					if (disposing && BaseStream != null) {
						BaseStream.Dispose();
					}
				} finally {
					BaseStream = null;
					base.Dispose(disposing);
				}
			}
		}

		void EnsureNotDisposed()
		{
			if (BaseStream == null) {
				throw new ObjectDisposedException(null);
			}
		}

		public override void Flush()
		{
			EnsureNotDisposed();
			BaseStream.Flush();
		}

		public override long Seek(long offset, SeekOrigin origin)
		{
			throw new NotSupportedException();
		}

		public override void SetLength(long value)
		{
			throw new NotSupportedException();
		}

		public override int Read(byte[] buffer, int offset, int count)
		{
			throw new NotSupportedException();
		}

		public override bool CanRead
		{
			get
			{
				return false;
			}
		}

		public override bool CanSeek
		{
			get
			{
				return false;
			}
		}

		public override bool CanWrite
		{
			get
			{
				return BaseStream != null && BaseStream.CanWrite;
			}
		}

		public override long Length
		{
			get
			{
				throw new NotSupportedException();
			}
		}

		public override long Position
		{
			get
			{
				throw new NotSupportedException();
			}
			set
			{
				throw new NotSupportedException();
			}
		}
	}
}
